"""
    Bot functions to generate and post a comments.

    Instructions to file with comments:
        one line - one comment.

    Example:
        lol
        kek

"""

import random
import os
import io

def get_random_comment_from_file(file_path):
    with io.open(file_path, "r", encoding="utf8") as f:
        content = f.readlines()
        return random.choice(content).strip()

def get_comment(bot, comment_base_file=None):
    """
        Generates comment.
        If comment_base_file argunment is passed, it uses the lines from file
        as comments to return.

        TODO: generate more way to create comments.
    """
    if comment_base_file is not None:
        if os.path.exists(comment_base_file):
            return get_random_comment_from_file(comment_base_file)
        else:
            print ("Can't find your file with comments.")
            return "lol"
    else:
        return "lol"



#!/usr/bin/env python
import os
import sys
import getpass

SECRET_FILE = "secret.txt"

def get_credentials():
    "Returns login and password stored in SECRET_FILE"
    while not check_secret():
        pass
    with open(SECRET_FILE, "r") as f:
        login = f.readline().strip()
        password = f.readline().strip()
    return login, password


def check_secret():
    while True:
        if os.path.exists(SECRET_FILE):
            with open(SECRET_FILE, "r") as f:
                try:
                    login = f.readline().strip()
                    password = f.readline().strip()
                    if len(login) < 4 or len(password) < 6:
                        print("Data in 'secret.txt' file is invalid. "
                              "We will delete it and try again.")
                        os.remove(SECRET_FILE)
                    else:
                        return True
                except IOError:
                    print("Your file is broken. We will delete it "
                          "and try again.")
                    os.remove(SECRET_FILE)
        else:
            with open(SECRET_FILE, "w") as f:
                print("We need to create a text file 'secret.txt' where "
                      "we will store your login and password from Instagram.")
                print("Don't worry. It will be stored locally.")
                print("Enter your login: ")
                f.write(str(sys.stdin.readline()))
                print("Enter your password: ")
                f.write(getpass.getpass())

def delete_credentials():
    if os.path.exists(SECRET_FILE):
        os.remove(SECRET_FILE)


if __name__ == "__main__":
    check_secret()



from . import config

def fbUserSearch(self, query):
    query = self.SendRequest('fbsearch/topsearch/?context=blended&query='+ str(query) +'&rank_token='+ str(self.rank_token))
    return query

def searchUsers(self, query):
    query = self.SendRequest('users/search/?ig_sig_key_version='+ str(config.SIG_KEY_VERSION)
            +'&is_typeahead=true&query='+ str(query) +'&rank_token='+ str(self.rank_token))
    return query

def searchUsername(self, usernameName):
    query = self.SendRequest('users/'+ str(usernameName) +'/usernameinfo/')
    return query

def searchTags(self, query):
    query = self.SendRequest('tags/search/?is_typeahead=true&q='+ str(query) +'&rank_token='+ str(self.rank_token))
    return query

def searchLocation(self, query):
    locationFeed = self.SendRequest('fbsearch/places/?rank_token='+ str(self.rank_token) +'&query=' + str(query))
    return locationFeed



"""
    Like-and-follow-users methods
"""

import time
import random
from tqdm import tqdm

def like_and_follow(bot, user_id, nlikes=3):
    """
        Likes and follows <user_id>. Puts <nlikes> in his media.
    """
    bot.like_user_id(user_id, amount=nlikes)
    bot.follow(user_id)
    return True

def like_and_follow_media_likers(bot, media, nlikes=3):
    """
        Takes input media. Takes the list of the likers.
        Follows liker and like <nlikes> of his last posted medias.

        args:
            bot - instance of instabot.Bot() class
            nlikes - number of likes to put on each user's media
    """
    print ("Going to like & follow users who liked %s media" % media)
    bot.getMediaLikers(media)
    media_likers = [item['pk'] for item in bot.LastJson["users"]]
    print ("  I have %d likers of media." % len(media_likers))
    for user in tqdm(media_likers, desc="Media likers"):
        bot.like_and_follow(user)
        time.sleep(10 + 20 * random.random())
    return True

def like_and_follow_your_feed_likers(bot, nlikes=3):
    """
        Takes your last media. Takes the list of the likers.
        Follows liker and like <nlikes> of his last posted medias.

        args:
            bot - instance of instabot.Bot() class
            nlikes - number of likes to put on each user's media
    """
    print ("Going to like & follow users who like your last media")
    bot.getSelfUserFeed()
    last_media = bot.LastJson["items"][0]["pk"]
    return like_and_follow_media_likers(bot, last_media, nlikes=3)



import json

def removeProfilePicture(self):
    data = json.dumps({
    '_uuid'        : self.uuid,
    '_uid'         : self.username_id,
    '_csrftoken'   : self.token
    })
    return self.SendRequest('accounts/remove_profile_picture/', self.generateSignature(data))

def setPrivateAccount(self):
    data = json.dumps({
    '_uuid'        : self.uuid,
    '_uid'         : self.username_id,
    '_csrftoken'   : self.token
    })
    return self.SendRequest('accounts/set_private/', self.generateSignature(data))

def setPublicAccount(self):
    data = json.dumps({
    '_uuid'        : self.uuid,
    '_uid'         : self.username_id,
    '_csrftoken'   : self.token
    })
    return self.SendRequest('accounts/set_public/', self.generateSignature(data))

def setNameAndPhone(self, name = '', phone = ''):
    data = json.dumps({
    '_uuid'         : self.uuid,
    '_uid'          : self.username_id,
    'first_name'    : name,
    'phone_number'  : phone,
    '_csrftoken'    : self.token
    })
    return self.SendRequest('accounts/set_phone_and_name/', self.generateSignature(data))

def getProfileData(self):
    data = json.dumps({
    '_uuid'        : self.uuid,
    '_uid'         : self.username_id,
    '_csrftoken'   : self.token
    })
    return self.SendRequest('accounts/current_user/?edit=true', self.generateSignature(data))

def editProfile(self, url, phone, first_name, biography, email, gender):
    data = json.dumps({
    '_uuid'        : self.uuid,
    '_uid'         : self.username_id,
    '_csrftoken'   : self.token,
    'external_url' : url,
    'phone_number' : phone,
    'username'     : self.username,
    'full_name'    : first_name,
    'biography'    : biography,
    'email'        : email,
    'gender'       : gender,
    })
    return self.SendRequest('accounts/edit_profile/', self.generateSignature(data))



"""
    Work with
        whitelist, blacklist,
        seleb account,
"""

import os

def read_list(file_path):
    """
        Reads whitelist/blacklist users from input file.
        Returns the list if file items
    """
    try:
        if not os.path.exists(file_path):
            print ("file %s does not exist." % file_path)
            return False
        with open(file_path, "r") as f:
            content = f.readlines()
            content = [item.strip() for item in content if len(item.strip()) > 0]
            return content
    except:
        return False

def add_whitelist(bot, file_path):
    bot.whitelist = read_list(file_path)
    return not not bot.whitelist

def add_blacklist(bot, file_path):
    bot.blacklist = read_list(file_path)
    return not not bot.blacklist

def get_media_owner(bot, media_id):
    bot.mediaInfo(media_id)
    try:
        return bot.LastJson["items"][0]["user"]["pk"]
    except:
        return False

def check_media(bot, media_id):
    return check_user(bot, get_media_owner(bot, media_id))

def check_user(bot, user_id):
    """
        Decide should you interract with that user_id or not.
        Decision based on
            1) Following/followers ratio
            2) Approved account
            3) Black-whitelist
            4) Closed accs
    """
    if not user_id:
        return True
    if user_id in bot.whitelist:
        return True
    if user_id in bot.blacklist:
        return False
    return True



import struct
import imghdr
import time
import json

from requests_toolbelt import MultipartEncoder

from . import config

def configurePhoto(self, upload_id, photo, caption = ''):
    (w,h) = getImageSize(photo)
    data = json.dumps({
    '_csrftoken'    : self.token,
    'media_folder'  : 'Instagram',
    'source_type'   : 4,
    '_uid'          : self.username_id,
    '_uuid'         : self.uuid,
    'caption'       : caption,
    'upload_id'     : upload_id,
    'device'        : config.DEVICE_SETTINTS,
    'edits'         : {
        'crop_original_size': [w * 1.0, h * 1.0],
        'crop_center'       : [0.0, 0.0],
        'crop_zoom'         : 1.0
    },
    'extra'         : {
        'source_width'  : w,
        'source_height' : h,
    }})
    return self.SendRequest('media/configure/?', self.generateSignature(data))

def uploadPhoto(self, photo, caption = None, upload_id = None):
    if upload_id is None:
        upload_id = str(int(time.time() * 1000))
    data = {
    'upload_id'         : upload_id,
    '_uuid'             : self.uuid,
    '_csrftoken'        : self.token,
    'image_compression' : '{"lib_name":"jt","lib_version":"1.3.0","quality":"87"}',
    'photo'             : ('pending_media_%s.jpg'%upload_id, open(photo, 'rb'), 'application/octet-stream', {'Content-Transfer-Encoding':'binary'})
    }
    m = MultipartEncoder(data, boundary=self.uuid)
    self.s.headers.update ({'X-IG-Capabilities' : '3Q4=',
                            'X-IG-Connection-Type' : 'WIFI',
                            'Cookie2' : '$Version=1',
                            'Accept-Language' : 'en-US',
                            'Accept-Encoding' : 'gzip, deflate',
                            'Content-type': m.content_type,
                            'Connection' : 'close',
                            'User-Agent' : config.USER_AGENT})
    response = self.s.post(config.API_URL + "upload/photo/", data=m.to_string())
    if response.status_code == 200:
        if self.configure(upload_id, photo, caption):
            self.expose()
    return False

def getImageSize(fname):
    with open(fname, 'rb') as fhandle:
        head = fhandle.read(24)
        if len(head) != 24:
            raise RuntimeError("Invalid Header")
        if imghdr.what(fname) == 'png':
            check = struct.unpack('>i', head[4:8])[0]
            if check != 0x0d0a1a0a:
                raise RuntimeError("PNG: Invalid check")
            width, height = struct.unpack('>ii', head[16:24])
        elif imghdr.what(fname) == 'gif':
            width, height = struct.unpack('<HH', head[6:10])
        elif imghdr.what(fname) == 'jpeg':
            fhandle.seek(0) # Read 0xff next
            size = 2
            ftype = 0
            while not 0xc0 <= ftype <= 0xcf:
                fhandle.seek(size, 1)
                byte = fhandle.read(1)
                while ord(byte) == 0xff:
                    byte = fhandle.read(1)
                ftype = ord(byte)
                size = struct.unpack('>H', fhandle.read(2))[0] - 2
            # We are at a SOFn block
            fhandle.seek(1, 1)  # Skip `precision' byte.
            height, width = struct.unpack('>HH', fhandle.read(4))
        else:
            raise RuntimeError("Unsupported format")
        return width, height



import time
import random
import pickle
from datetime import datetime

class Checkpoint(object):
    """
        Checkpoint for instabot.Bot which stores:
            .following (list of user_ids)
            .followers (list of user_ids)
            .date (of checkpoint creation)
    """
    def __init__(self):
        self.following = []
        self.followers = []
        self.date = None

    def fill(self, bot):
        self.following = [item["pk"] for item in bot.getTotalSelfFollowings()]
        self.followers = [item["pk"] for item in bot.getTotalSelfFollowers()]
        self.date = datetime.now()


def save_checkpoint(bot, path=None):
    """
        Saves bot's checkpoint:
            followers
            following
            date
        Returns the name of saved file
    """
    print ("Saving checkpoint:")
    if path is None:
        path = datetime.now().strftime("bot_cp_%Y-%m-%d_%H-%M")

    cp = Checkpoint()
    cp.fill(bot)

    with open(path, 'wb') as f:
        pickle.dump(cp, f, -1)
    bot.last_checkpoint_path = path
    print ("  Done.")
    return path

def load_checkpoint(bot, path):
    """
        Loads bot's checkpoint
        Returns Checkpoint object
    """
    print ("Loading checkpoint:")
    try:
        with open(path, 'rb') as f:
            cp = pickle.load(f)
        if isinstance(cp, Checkpoint):
            return cp
        else:
            print ("  This is not checkpoint file.")
    except:
        print ("  File not found.")
    return None

def checkpoint_following_diff(bot, cp):
    """
        Returns user_ids of users that you follow now
        but didn't follow at checkpoint time.
    """
    print ("Getting checkpoint following difference.")
    current_following = [item["pk"] for item in bot.getTotalSelfFollowings()]
    old_following = cp.following
    return list(set(current_following) - set(old_following))

def checkpoint_followers_diff(bot, cp):
    """
        Returns user_ids of users that follows you now
        but didn't follow you at checkpoint time.
    """
    print ("Getting checkpoint followers difference.")
    current_followers = [item["pk"] for item in bot.getTotalSelfFollowers()]
    old_following = cp.followers
    return list(set(current_followers) - set(old_followers))

def load_last_checkpoint(bot):
    return bot.load_checkpoint(bot.last_checkpoint_path)

def revert_to_checkpoint(bot, cp):
    return bot.unfollow_users(bot.checkpoint_following_diff(cp))



"""
    All methods must return media_ids that can be
    passed into e.g. like() or comment() functions.
"""

def filter_not_liked(media_items, log=False):
    not_liked_medias = [item["pk"] for item in media_items if not item["has_liked"]]
    if log:
        print ("  Recieved: %d. Already liked: %d." % (
                            len(media_items),
                            len(media_items) - len(not_liked_medias)
                            )
        )
    return not_liked_medias

def get_timeline_medias(bot):
    if not bot.getTimelineFeed():
        print ("  Error while getting timeline feed")
        return False
    return filter_not_liked(bot.LastJson["items"])

def get_user_medias(bot, user_id):
    bot.getUserFeed(user_id)
    if bot.LastJson["status"] == 'fail':
        print ("  This is a closed account")
        return False
    return filter_not_liked(bot.LastJson["items"])

def get_hashtag_medias(bot, hashtag):
    if not bot.getHashtagFeed(hashtag):
        print ("Error while getting hashtag feed")
        return False
    return filter_not_liked(bot.LastJson["items"])



"""
Configuration file of the project
"""

API_URL = 'https://i.instagram.com/api/v1/'
DEVICE_SETTINTS = {
'manufacturer'      : 'Xiaomi',
'model'             : 'HM 1SW',
'android_version'   : 18,
'android_release'   : '4.3'
}
USER_AGENT = 'Instagram 9.2.0 Android ({android_version}/{android_release}; 320dpi; 720x1280; {manufacturer}; {model}; armani; qcom; en_US)'.format(**DEVICE_SETTINTS)
IG_SIG_KEY = '012a54f51c49aa8c5c322416ab1410909add32c966bbaa0fe3dc58ac43fd7ede'
EXPERIMENTS = 'ig_android_progressive_jpeg,ig_creation_growth_holdout,ig_android_report_and_hide,ig_android_new_browser,ig_android_enable_share_to_whatsapp,ig_android_direct_drawing_in_quick_cam_universe,ig_android_huawei_app_badging,ig_android_universe_video_production,ig_android_asus_app_badging,ig_android_direct_plus_button,ig_android_ads_heatmap_overlay_universe,ig_android_http_stack_experiment_2016,ig_android_infinite_scrolling,ig_fbns_blocked,ig_android_white_out_universe,ig_android_full_people_card_in_user_list,ig_android_post_auto_retry_v7_21,ig_fbns_push,ig_android_feed_pill,ig_android_profile_link_iab,ig_explore_v3_us_holdout,ig_android_histogram_reporter,ig_android_anrwatchdog,ig_android_search_client_matching,ig_android_high_res_upload_2,ig_android_new_browser_pre_kitkat,ig_android_2fac,ig_android_grid_video_icon,ig_android_white_camera_universe,ig_android_disable_chroma_subsampling,ig_android_share_spinner,ig_android_explore_people_feed_icon,ig_explore_v3_android_universe,ig_android_media_favorites,ig_android_nux_holdout,ig_android_search_null_state,ig_android_react_native_notification_setting,ig_android_ads_indicator_change_universe,ig_android_video_loading_behavior,ig_android_black_camera_tab,liger_instagram_android_univ,ig_explore_v3_internal,ig_android_direct_emoji_picker,ig_android_prefetch_explore_delay_time,ig_android_business_insights_qe,ig_android_direct_media_size,ig_android_enable_client_share,ig_android_promoted_posts,ig_android_app_badging_holdout,ig_android_ads_cta_universe,ig_android_mini_inbox_2,ig_android_feed_reshare_button_nux,ig_android_boomerang_feed_attribution,ig_android_fbinvite_qe,ig_fbns_shared,ig_android_direct_full_width_media,ig_android_hscroll_profile_chaining,ig_android_feed_unit_footer,ig_android_media_tighten_space,ig_android_private_follow_request,ig_android_inline_gallery_backoff_hours_universe,ig_android_direct_thread_ui_rewrite,ig_android_rendering_controls,ig_android_ads_full_width_cta_universe,ig_video_max_duration_qe_preuniverse,ig_android_prefetch_explore_expire_time,ig_timestamp_public_test,ig_android_profile,ig_android_dv2_consistent_http_realtime_response,ig_android_enable_share_to_messenger,ig_explore_v3,ig_ranking_following,ig_android_pending_request_search_bar,ig_android_feed_ufi_redesign,ig_android_video_pause_logging_fix,ig_android_default_folder_to_camera,ig_android_video_stitching_7_23,ig_android_profanity_filter,ig_android_business_profile_qe,ig_android_search,ig_android_boomerang_entry,ig_android_inline_gallery_universe,ig_android_ads_overlay_design_universe,ig_android_options_app_invite,ig_android_view_count_decouple_likes_universe,ig_android_periodic_analytics_upload_v2,ig_android_feed_unit_hscroll_auto_advance,ig_peek_profile_photo_universe,ig_android_ads_holdout_universe,ig_android_prefetch_explore,ig_android_direct_bubble_icon,ig_video_use_sve_universe,ig_android_inline_gallery_no_backoff_on_launch_universe,ig_android_image_cache_multi_queue,ig_android_camera_nux,ig_android_immersive_viewer,ig_android_dense_feed_unit_cards,ig_android_sqlite_dev,ig_android_exoplayer,ig_android_add_to_last_post,ig_android_direct_public_threads,ig_android_prefetch_venue_in_composer,ig_android_bigger_share_button,ig_android_dv2_realtime_private_share,ig_android_non_square_first,ig_android_video_interleaved_v2,ig_android_follow_search_bar,ig_android_last_edits,ig_android_video_download_logging,ig_android_ads_loop_count_universe,ig_android_swipeable_filters_blacklist,ig_android_boomerang_layout_white_out_universe,ig_android_ads_carousel_multi_row_universe,ig_android_mentions_invite_v2,ig_android_direct_mention_qe,ig_android_following_follower_social_context'
SIG_KEY_VERSION = '4'



import time
import random
import json

def unfollow_non_followers(bot):
    print ("Unfollow non followers:")
    followings = set([item["pk"] for item in bot.getTotalSelfFollowings()])
    print ("  You follow %d users." % len(followings))
    followers = set([item["pk"] for item in bot.getTotalSelfFollowers()])
    print ("  You are followed by %d users." % len(followers))
    diff = followings - followers
    print ("  %d users don't follow you back." % len(diff))
    bot.unfollow_users(list(diff))



import time
import datetime
import random
import atexit
import signal
from tqdm import tqdm

from .. import API

from . import limits

from .bot_get_medias import get_timeline_medias
from .bot_get_medias import get_user_medias
from .bot_get_medias import get_hashtag_medias

from .bot_like_feed import like_timeline
from .bot_like_feed import like_user_id
from .bot_like_feed import like_hashtag
from .bot_unfollow_non_followers import unfollow_non_followers

from .bot_checkpoint import save_checkpoint
from .bot_checkpoint import load_checkpoint
from .bot_checkpoint import checkpoint_followers_diff
from .bot_checkpoint import checkpoint_following_diff
from .bot_checkpoint import load_last_checkpoint
from .bot_checkpoint import revert_to_checkpoint

from .bot_like_and_follow import like_and_follow
from .bot_like_and_follow import like_and_follow_media_likers
from .bot_like_and_follow import like_and_follow_your_feed_likers

from .bot_comment import get_comment

from .bot_filter import read_list
from .bot_filter import get_media_owner
from .bot_filter import check_media
from .bot_filter import check_user

class Bot(API):
    def __init__(self,
                 whitelist=False,
                 blacklist=False):
        super(self.__class__, self).__init__()
        self.total_liked = 0
        self.total_unliked = 0
        self.total_followed = 0
        self.total_unfollowed = 0
        self.total_commented = 0
        self.start_time = datetime.datetime.now()
        if whitelist:
            self.whitelist = read_list(whitelist)
            print ("Size of whitelist: %d" % len(self.whitelist))
        if blacklist:
            self.blacklist = read_list(blacklist)
            print ("Size of blacklist: %d" % len(self.blacklist))

        signal.signal(signal.SIGTERM, self.logout)
        atexit.register(self.logout)

    def logout(self):
        super(self.__class__, self).logout()
        print ("Bot stopped. "
               "Worked: %s" % (datetime.datetime.now() - self.start_time))
        if self.total_liked:
            print ("  Total liked: %d" % self.total_liked)
        if self.total_unliked:
            print ("  Total unliked: %d" % self.total_unliked)
        if self.total_followed:
            print ("  Total followed: %d" % self.total_followed)
        if self.total_unfollowed:
            print ("  Total unfollowed: %d" % self.total_unfollowed)
        if self.total_commented:
            print ("  Total commented: %d" % self.total_commented)

    def like(self, media_id):
        if not self.check_media(media_id):
            return False
        if super(self.__class__, self).like(media_id):
            self.total_liked += 1
            return True
        return False

    def unlike(self, media_id):
        if not self.check_media(media_id):
            return False
        if super(self.__class__, self).unlike(media_id):
            self.total_unliked += 1
            return True
        return False

    def follow(self, user_id):
        if not self.check_user(user_id):
            return False
        if super(self.__class__, self).follow(user_id):
            self.total_followed += 1
            return True
        return False

    def unfollow(self, user_id):
        if not self.check_user(user_id):
            return False
        if super(self.__class__, self).unfollow(user_id):
            self.total_unfollowed += 1
            return True
        return False

    def comment(self, media_id, comment_text):
        if not self.check_media(media_id):
            return False
        if super(self.__class__, self).comment(media_id, comment_text):
            self.total_commented += 1
            return True
        return False

    def like_medias(self, medias):
        """ medias - list of ["pk"] fields of response """
        print ("    Going to like %d medias." % (len(medias)))
        total_liked = 0
        for media in tqdm(medias):
            if self.like(media):
                total_liked += 1
            else:
                pass
            time.sleep(10 * random.random())
        print ("    DONE: Total liked %d medias. " % total_liked)
        return True

    def follow_users(self, user_ids):
        """ user_ids - list of user_id to follow """
        print ("    Going to follow %d users." % len(user_ids))
        total_followed = 0
        for user_id in tqdm(user_ids):
            if self.follow(user_id):
                total_followed += 1
            else:
                pass
            time.sleep(15 + 30 * random.random())
        print ("    DONE: Total followed %d users. " % total_followed)
        return True

    def unfollow_users(self, user_ids):
        """ user_ids - list of user_id to unfollow """
        print ("    Going to unfollow %d users." % len(user_ids))
        total_unfollowed = 0
        for user_id in tqdm(user_ids):
            if self.unfollow(user_id):
                total_unfollowed += 1
            else:
                pass
            time.sleep(15 + 30 * random.random())
        print ("    DONE: Total unfollowed %d users. " % total_unfollowed)
        return True

    def get_timeline_medias(self):
        return get_timeline_medias(self)

    def get_user_medias(self, user_id):
        return get_user_medias(self, user_id)

    def like_timeline(self, amount=None):
        return like_timeline(self, amount)

    def like_user_id(self, user_id, amount=None):
        return like_user_id(self, user_id, amount)

    def unfollow_non_followers(self):
        return unfollow_non_followers(self)

    def like_hashtag(self, tag, amount=None):
        return like_hashtag(self, tag, amount)

    def save_checkpoint(self):
        return save_checkpoint(self)

    def load_checkpoint(self, path):
        return load_checkpoint(self, path)

    def checkpoint_followers_diff(self, cp):
        return checkpoint_followers_diff(self, cp)

    def checkpoint_following_diff(self, cp):
        return checkpoint_following_diff(self, cp)

    def load_last_checkpoint(self):
        return load_last_checkpoint(self)

    def revert_to_checkpoint(self, cp):
        return revert_to_checkpoint(self, cp)

    def like_and_follow(self, user_id, nlikes=3):
        return like_and_follow(self, user_id, nlikes)

    def like_and_follow_media_likers(self, media, nlikes=3):
        return like_and_follow_media_likers(self, media, nlikes)

    def like_and_follow_your_feed_likers(self, nlikes=3):
        return like_and_follow_your_feed_likers(self, nlikes)

    def get_comment(self, comment_base_file=None):
        return get_comment(self, comment_base_file)

    def add_whitelist(self, file_path):
        return add_whitelist(self, file_path)

    def add_blacklist(self, file_path):
        return add_blacklist(self, file_path)

    def get_media_owner(self, media):
        return get_media_owner(self, media)

    def check_media(self, media):
        return check_media(self, media)

    def check_user(self, user):
        return check_user(self, user)



import time
import random

def like_timeline(bot, amount=None):
    """ Likes last 8 medias from timeline feed """
    print ("Liking timeline feed:")
    medias = bot.get_timeline_medias()[:amount]
    return bot.like_medias(medias)

def like_user_id(bot, user_id, amount=None):
    """ Likes last user_id's medias """
    if not user_id:
        return False
    print ("Liking user_%s's feed:" % user_id)
    medias = bot.get_user_medias(user_id)
    if not medias:
        print ("  Can't like user: account is closed!")
        return False
    return bot.like_medias(medias[:amount])

def like_hashtag(bot, hashtag, amount=None):
    print ("Going to like medias by %s hashtag" % hashtag)
    medias = bot.get_hashtag_medias(bot, hashtag)
    return bot.like_medias(medias[:amount])



import requests
import random
import json
import hashlib
import hmac
import urllib
import uuid
import sys

#The urllib library was split into other modules from Python 2 to Python 3
if sys.version_info.major == 3:
    import urllib.parse

from . import config
from .api_photo import configurePhoto
from .api_photo import uploadPhoto

from .api_search import fbUserSearch
from .api_search import searchUsers
from .api_search import searchUsername
from .api_search import searchTags
from .api_search import searchLocation

from .api_profile import removeProfilePicture
from .api_profile import setPrivateAccount
from .api_profile import setPublicAccount
from .api_profile import getProfileData
from .api_profile import editProfile
from .api_profile import setNameAndPhone

from .prepare import get_credentials
from .prepare import delete_credentials

class API(object):
    def __init__(self):
        self.isLoggedIn = False
        self.LastResponse = None

    def setUser(self, username, password):
        self.username = username
        self.password = password
        self.uuid = self.generateUUID(True)

    def login(self, username = None, password = None, force = False):
        if username is None or password is None:
            username, password = get_credentials()

        m = hashlib.md5()
        m.update(username.encode('utf-8') + password.encode('utf-8'))
        self.device_id = self.generateDeviceId(m.hexdigest())
        self.setUser(username, password)

        if (not self.isLoggedIn or force):
            self.session = requests.Session()
            # if you need proxy make something like this:
            # self.session.proxies = {"https" : "http://proxyip:proxyport"}
            if (self.SendRequest('si/fetch_headers/?challenge_type=signup&guid=' + self.generateUUID(False), None, True)):

                data = {'phone_id'   : self.generateUUID(True),
                        '_csrftoken' : self.LastResponse.cookies['csrftoken'],
                        'username'   : self.username,
                        'guid'       : self.uuid,
                        'device_id'  : self.device_id,
                        'password'   : self.password,
                        'login_attempt_count' : '0'}

                if self.SendRequest('accounts/login/', self.generateSignature(json.dumps(data)), True):
                    self.isLoggedIn = True
                    self.username_id = self.LastJson["logged_in_user"]["pk"]
                    self.rank_token = "%s_%s" % (self.username_id, self.uuid)
                    self.token = self.LastResponse.cookies["csrftoken"]

                    # self.syncFeatures()
                    # self.autoCompleteUserList()
                    # self.getTimelineFeed()
                    # self.getv2Inbox()
                    # self.getRecentActivity()
                    print ("Login success!\n")
                    return True
                else:
                    print ("Login or password is incorrect.")
                    delete_credentials()
                    exit()

    def logout(self):
        if not self.isLoggedIn:
            return True
        return self.SendRequest('accounts/logout/')

    def SendRequest(self, endpoint, post = None, login = False):
        if (not self.isLoggedIn and not login):
            raise Exception("Not logged in!\n")

        self.session.headers.update({'Connection' : 'close',
                                'Accept' : '*/*',
                                'Content-type' : 'application/x-www-form-urlencoded; charset=UTF-8',
                                'Cookie2' : '$Version=1',
                                'Accept-Language' : 'en-US',
                                'User-Agent' : config.USER_AGENT})

        if (post != None): # POST
            response = self.session.post(config.API_URL + endpoint, data=post) # , verify=False
        else: # GET
            response = self.session.get(config.API_URL + endpoint) # , verify=False

        if response.status_code == 200:
            self.LastResponse = response
            self.LastJson = json.loads(response.text)
            return True
        else:
            print ("Request return " + str(response.status_code) + " error!")
            # for debugging
            try:
                self.LastResponse = response
                self.LastJson = json.loads(response.text)
            except:
                pass
            return False

    def syncFeatures(self):
        data = json.dumps({
        '_uuid'         : self.uuid,
        '_uid'          : self.username_id,
        'id'            : self.username_id,
        '_csrftoken'    : self.token,
        'experiments'   : config.EXPERIMENTS
        })
        return self.SendRequest('qe/sync/', self.generateSignature(data))

    def autoCompleteUserList(self):
        return self.SendRequest('friendships/autocomplete_user_list/')

    def getTimelineFeed(self):
        """ Returns 8 medias from timeline feed of logged user """
        return self.SendRequest('feed/timeline/')

    def megaphoneLog(self):
        return self.SendRequest('megaphone/log/')

    def expose(self):
        data = json.dumps({
        '_uuid'        : self.uuid,
        '_uid'         : self.username_id,
        'id'           : self.username_id,
        '_csrftoken'   : self.token,
        'experiment'   : 'ig_android_profile_contextual_feed'
        })
        return self.SendRequest('qe/expose/', self.generateSignature(data))

    def uploadPhoto(self, photo, caption = None, upload_id = None):
        return uploadPhoto(self, photo, caption, upload_id)

    def configurePhoto(self, upload_id, photo, caption = ''):
        return configurePhoto(self, upload_id, photo, caption)

    def editMedia(self, mediaId, captionText = ''):
        data = json.dumps({
        '_uuid'        : self.uuid,
        '_uid'         : self.username_id,
        '_csrftoken'   : self.token,
        'caption_text' : captionText
        })
        return self.SendRequest('media/'+ str(mediaId) +'/edit_media/', self.generateSignature(data))

    def removeSelftag(self, mediaId):
        data = json.dumps({
        '_uuid'        : self.uuid,
        '_uid'         : self.username_id,
        '_csrftoken'   : self.token
        })
        return self.SendRequest('media/'+ str(mediaId) +'/remove/', self.generateSignature(data))

    def mediaInfo(self, mediaId):
        data = json.dumps({
        '_uuid'        : self.uuid,
        '_uid'         : self.username_id,
        '_csrftoken'   : self.token,
        'media_id'     : mediaId
        })
        return self.SendRequest('media/'+ str(mediaId) +'/info/', self.generateSignature(data))

    def deleteMedia(self, mediaId):
        data = json.dumps({
        '_uuid'        : self.uuid,
        '_uid'         : self.username_id,
        '_csrftoken'   : self.token,
        'media_id'     : mediaId
        })
        return self.SendRequest('media/'+ str(mediaId) +'/delete/', self.generateSignature(data))

    def changePassword(self, newPassword):
        data = json.dumps({
        '_uuid'        : self.uuid,
        '_uid'         : self.username_id,
        '_csrftoken'   : self.token,
        'old_password'  : self.password,
        'new_password1' : newPassword,
        'new_password2' : newPassword
        })
        return self.SendRequest('accounts/change_password/', self.generateSignature(data))

    def explore(self):
        return self.SendRequest('discover/explore/')

    def comment(self, mediaId, commentText):
        data = json.dumps({
        '_uuid'        : self.uuid,
        '_uid'         : self.username_id,
        '_csrftoken'   : self.token,
        'comment_text' : commentText
        })
        return self.SendRequest('media/'+ str(mediaId) +'/comment/', self.generateSignature(data))

    def deleteComment(self, mediaId, commentId):
        data = json.dumps({
        '_uuid'        : self.uuid,
        '_uid'         : self.username_id,
        '_csrftoken'   : self.token
        })
        return self.SendRequest('media/'+ str(mediaId) +'/comment/'+ str(commentId) +'/delete/', self.generateSignature(data))

    def removeProfilePicture(self):
        return removeProfilePicture(self)

    def setPrivateAccount(self):
        return setPrivateAccount(self)

    def setPublicAccount(self):
        return setPublicAccount(self)

    def getProfileData(self):
        return getProfileData(self)

    def editProfile(self, url, phone, first_name, biography, email, gender):
        return editProfile(self, url, phone, first_name, biography, email, gender)

    def getUsernameInfo(self, usernameId):
        return self.SendRequest('users/'+ str(usernameId) +'/info/')

    def getSelfUsernameInfo(self):
        return self.getUsernameInfo(self.username_id)

    def getRecentActivity(self):
        activity = self.SendRequest('news/inbox/?')
        return activity

    def getFollowingRecentActivity(self):
        activity = self.SendRequest('news/?')
        return activity

    def getv2Inbox(self):
        inbox = self.SendRequest('direct_v2/inbox/?')
        return inbox

    def getUserTags(self, usernameId):
        tags = self.SendRequest('usertags/'+ str(usernameId) +'/feed/?rank_token='+ str(self.rank_token) +'&ranked_content=true&')
        return tags

    def getSelfUserTags(self):
        return self.getUserTags(self.username_id)

    def tagFeed(self, tag):
        userFeed = self.SendRequest('feed/tag/'+ str(tag) +'/?rank_token=' + str(self.rank_token) + '&ranked_content=true&')
        return userFeed

    def getMediaLikers(self, mediaId):
        likers = self.SendRequest('media/'+ str(mediaId) +'/likers/?')
        return likers

    def getGeoMedia(self, usernameId):
        locations = self.SendRequest('maps/user/'+ str(usernameId) +'/')
        return locations

    def getSelfGeoMedia(self):
        return self.getGeoMedia(self.username_id)

    def fbUserSearch(self, query):
        return fbUserSearch(self, query)

    def searchUsers(self, query):
        return searchUsers(self, query)

    def searchUsername(self, username):
        return searchUsername(self, username)

    def searchTags(self, query):
        return searchTags(self, query)

    def searchLocation(self, query):
        return searchLocation(self, query)

    def syncFromAdressBook(self, contacts):
        return self.SendRequest('address_book/link/?include=extra_display_name,thumbnails', "contacts=" + json.dumps(contacts))

    def getTimeline(self):
        query = self.SendRequest('feed/timeline/?rank_token='+ str(self.rank_token) +'&ranked_content=true&')
        return query

    def getUserFeed(self, usernameId, maxid = '', minTimestamp = None):
        query = self.SendRequest('feed/user/' + str(usernameId) + '/?max_id=' + str(maxid) + '&min_timestamp=' + str(minTimestamp)
            + '&rank_token='+ str(self.rank_token) +'&ranked_content=true')
        return query

    def getSelfUserFeed(self, maxid = '', minTimestamp = None):
        return self.getUserFeed(self.username_id, maxid, minTimestamp)

    def getHashtagFeed(self, hashtagString, maxid = ''):
        return self.SendRequest('feed/tag/'+hashtagString+'/?max_id='+str(maxid)+'&rank_token='+self.rank_token+'&ranked_content=true&')

    def getLocationFeed(self, locationId, maxid = ''):
        return self.SendRequest('feed/location/'+str(locationId)+'/?max_id='+maxid+'&rank_token='+self.rank_token+'&ranked_content=true&')

    def getPopularFeed(self):
        popularFeed = self.SendRequest('feed/popular/?people_teaser_supported=1&rank_token='+ str(self.rank_token) +'&ranked_content=true&')
        return popularFeed

    def getUserFollowings(self, usernameId, maxid = ''):
        return self.SendRequest('friendships/'+ str(usernameId) +'/following/?max_id='+ str(maxid)
            +'&ig_sig_key_version='+ config.SIG_KEY_VERSION +'&rank_token='+ self.rank_token)

    def getSelfUsersFollowing(self):
        return self.getUserFollowings(self.username_id)

    def getUserFollowers(self, usernameId, maxid = ''):
        if maxid == '':
            return self.SendRequest('friendships/'+ str(usernameId) +'/followers/?rank_token='+ self.rank_token)
        else:
            return self.SendRequest('friendships/'+ str(usernameId) +'/followers/?rank_token='+ self.rank_token + '&max_id='+ str(maxid))

    def getSelfUserFollowers(self):
        return self.getUserFollowers(self.username_id)

    def like(self, mediaId):
        data = json.dumps({
        '_uuid'         : self.uuid,
        '_uid'          : self.username_id,
        '_csrftoken'    : self.token,
        'media_id'      : mediaId
        })
        return self.SendRequest('media/'+ str(mediaId) +'/like/', self.generateSignature(data))

    def unlike(self, mediaId):
        data = json.dumps({
        '_uuid'         : self.uuid,
        '_uid'          : self.username_id,
        '_csrftoken'    : self.token,
        'media_id'      : mediaId
        })
        return self.SendRequest('media/'+ str(mediaId) +'/unlike/', self.generateSignature(data))

    def getMediaComments(self, mediaId):
        return self.SendRequest('media/'+ mediaId +'/comments/?')

    def setNameAndPhone(self, name = '', phone = ''):
        return setNameAndPhone(self, name, phone)

    def getDirectShare(self):
        return self.SendRequest('direct_share/inbox/?')

    def follow(self, userId):
        data = json.dumps({
        '_uuid'         : self.uuid,
        '_uid'          : self.username_id,
        'user_id'       : userId,
        '_csrftoken'    : self.token
        })
        return self.SendRequest('friendships/create/'+ str(userId) +'/', self.generateSignature(data))

    def unfollow(self, userId):
        data = json.dumps({
        '_uuid'         : self.uuid,
        '_uid'          : self.username_id,
        'user_id'       : userId,
        '_csrftoken'    : self.token
        })
        return self.SendRequest('friendships/destroy/'+ str(userId) +'/', self.generateSignature(data))

    def block(self, userId):
        data = json.dumps({
        '_uuid'         : self.uuid,
        '_uid'          : self.username_id,
        'user_id'       : userId,
        '_csrftoken'    : self.token
        })
        return self.SendRequest('friendships/block/'+ str(userId) +'/', self.generateSignature(data))

    def unblock(self, userId):
        data = json.dumps({
        '_uuid'         : self.uuid,
        '_uid'          : self.username_id,
        'user_id'       : userId,
        '_csrftoken'    : self.token
        })
        return self.SendRequest('friendships/unblock/'+ str(userId) +'/', self.generateSignature(data))

    def userFriendship(self, userId):
        data = json.dumps({
        '_uuid'         : self.uuid,
        '_uid'          : self.username_id,
        'user_id'       : userId,
        '_csrftoken'    : self.token
        })
        return self.SendRequest('friendships/show/'+ str(userId) +'/', self.generateSignature(data))

    def generateSignature(self, data):
        try:
            parsedData = urllib.parse.quote(data)
        except AttributeError:
            parsedData = urllib.quote(data)

        return 'ig_sig_key_version=' + config.SIG_KEY_VERSION + '&signed_body=' + hmac.new(config.IG_SIG_KEY.encode('utf-8'), data.encode('utf-8'), hashlib.sha256).hexdigest() + '.' + parsedData

    def generateDeviceId(self, seed):
        volatile_seed = "12345"
        m = hashlib.md5()
        m.update(seed.encode('utf-8') + volatile_seed.encode('utf-8'))
        return 'android-' + m.hexdigest()[:16]

    def generateUUID(self, uuid_type):
        generated_uuid = str(uuid.uuid4())
        if (uuid_type):
            return generated_uuid
        else:
            return generated_uuid.replace('-', '')

    def getLikedMedia(self,maxid=''):
        return self.SendRequest('feed/liked/?max_id='+str(maxid))

    def getTotalFollowers(self,usernameId):
        followers = []
        next_max_id = ''
        while 1:
            self.getUserFollowers(usernameId,next_max_id)
            temp = self.LastJson

            for item in temp["users"]:
                followers.append(item)

            if temp["big_list"] == False:
                return followers
            next_max_id = temp["next_max_id"]

    def getTotalFollowings(self,usernameId):
        followers = []
        next_max_id = ''
        while 1:
            self.getUserFollowings(usernameId,next_max_id)
            temp = self.LastJson

            for item in temp["users"]:
                followers.append(item)

            if temp["big_list"] == False:
                return followers
            next_max_id = temp["next_max_id"]

    def getTotalUserFeed(self, usernameId, minTimestamp = None):
        user_feed = []
        next_max_id = ''
        while 1:
            self.getUserFeed(usernameId, next_max_id, minTimestamp)
            temp = self.LastJson
            for item in temp["items"]:
                user_feed.append(item)
            if temp["more_available"] == False:
                return user_feed
            next_max_id = temp["next_max_id"]

    def getTotalSelfUserFeed(self, minTimestamp = None):
        return self.getTotalUserFeed(self.username_id, minTimestamp)

    def getTotalSelfFollowers(self):
        return self.getTotalFollowers(self.username_id)

    def getTotalSelfFollowings(self):
        return self.getTotalFollowings(self.username_id)

    def getTotalLikedMedia(self, scan_rate = 1):
        next_id = ''
        liked_items = []
        for _ in range(0,scan_rate):
            temp = self.getLikedMedia(next_id)
            temp = self.LastJson
            next_id = temp["next_max_id"]
            for item in temp["items"]:
                liked_items.append(item)
        return liked_items

# TODOs:

    def changeProfilePicture(self, photo):
        # TODO Instagram.php 705-775
        return False

    def direct_share(self, media_id, recipients, text = None):
        # TODO Instagram.php 420-490
        return False

    def buildBody(bodies, boundary):
        # TODO Instagram.php 1620-1645
        return False

    def backup(self):
        # TODO Instagram.php 1470-1485
        return False



from distutils.core import setup
setup(
  name = 'instabot',
  packages = ['instabot', 'instabot.bot', 'instabot.api'],
  version = '0.2.5.5',
  description = 'Cool Instagram bot scripts and API python wrapper.',
  author = 'Daniil Okhlopkov',
  author_email = 'ohld@icloud.com',
  url = 'https://github.com/ohld/instabot',
  download_url = 'https://github.com/ohld/instabot/tarball/0.2.5.5',
  keywords = ['instagram', 'bot', 'api'],
  classifiers = [],
  install_requires=['tqdm', 'requests-toolbelt', 'requests'],
)
